(function () {
    'use strict';

    angular
        .module('app.core')
        .provider('keyEvents', provideKeyEvents);

    /** @ngInject */
    function provideKeyEvents() {
        // I hold the list of functions that can be used to decorate the .is.
        // object with additional short-hand properties.
        var shortcuts = [
            function operator(event, is) {

                // Always add the lower-case key.
                is[String.fromCharCode(event.which).toLowerCase()] = true;

            }
        ];

        // Return the provider API.
        return ({
            // This is the core factory method for the service.
            $get: keyEventsFactory,

            // This is the provider-method that lets the developer add shortcut
            // operators.
            addShortcut: addShortcut
        });

        // PUBLIC METHODS.

        // I add a new shortcut operator that will be applied to every key event.
        function addShortcut(operator) {

            shortcuts.push(operator);

        }

        // FACTORY METHOD.

        // I provide the keyEvents service.
        function keyEventsFactory($document, KeyEventHandler) {

            // I hold the event handler queues in ascending priority order. As
            // such, they should be iterated over in reverse order.
            var eventHandlers = {
                keydown: [],
                keypress: [],
                keyup: []
            };

            // Setup the public API.
            var api = {
                handler: handler,
                off: off,
                on: on
            };

            // Return the public API.
            return ( api );


            // PUBLIC METHODS.


            // I create a new handler with the given priority. If the terminal
            // flag is set, no handlers at a lower priority will receive the
            // event, regardless of what this handler does to the event.
            function handler(priority, isTerminal) {

                return ( new KeyEventHandler(api, priority, ( isTerminal || false )) );

            }


            // I unbind the given event handler. This will unbind all matching
            // handlers at any priority.
            function off(eventType, handler) {

                deregisterHandler(eventType, handler);

            }


            // I bind a handler to the given event type at the given priority.
            // If the terminal flag is set, no other handlers at a lower priority
            // will receive the event.
            function on(eventType, handler, priority, isTerminal) {

                registerHandler(eventType, handler, priority, isTerminal);

            }


            // PRIVATE METHODS.


            // I remove the given handler from the given event type.
            function deregisterHandler(eventType, handler) {

                var handlers = eventHandlers[eventType];

                for (var i = ( handlers.length - 1 ); i >= 0; i--) {

                    if (handlers[i].handler === handler) {

                        handlers.splice(i, 1);

                    }

                }

                // If this particular event has no more event bindings, then
                // stop watching for it at the root level.
                if (!handlers.length) {

                    stopWatchingEvent(eventType);

                }

            }


            // I process the root key-event, passing it through queue of event
            // handlers implemented by the directives.
            function handleEvent(event) {
                var i, length;
                var handlers = eventHandlers[event.type];

                event.is = {};

                // Pass the event through the shortcut operators.
                for (i = 0, length = shortcuts.length; i < length; i++) {

                    shortcuts[i](event, event.is);

                }

                // Since the handlers are sorted in ascending priority, we need
                // to process the queue in reverse order.
                for (i = ( handlers.length - 1 ); i >= 0; i--) {

                    // If a handler returns an explicit false, kill the event.
                    if (handlers[i].handler(event) === false) {

                        event.stopImmediatePropagation();
                        event.preventDefault();
                        return ( false );

                    }

                    // If the handler is flagged as terminal, or if the handle
                    // explicitly stopped propagation, don't pass the event onto
                    // any other handlers.
                    if (handlers[i].isTerminal || event.isImmediatePropagationStopped()) {

                        break;

                    }

                }

            }


            // I register the given event-type handler at the given priority.
            function registerHandler(eventType, handler, priority, isTerminal) {

                var handlers = eventHandlers[eventType];

                var newItem = {
                    handler: handler,
                    priority: priority,
                    isTerminal: isTerminal
                };

                // If it's the first handler, just push it and start watching for
                // events on the document.
                if (!handlers.length) {

                    handlers.push(newItem);

                    return ( startWatchingEvent(eventType) );

                }

                // If it's the lowest priority handler, push it onto the bottom.
                if (handlers[0].priority > priority) {

                    return ( handlers.unshift(newItem) );

                }

                // Otherwise, add it in priority sort order.
                for (var i = ( handlers.length - 1 ); i >= 0; i--) {

                    if (handlers[i].priority <= priority) {

                        return ( handlers.splice(( i + 1 ), 0, newItem) );

                    }

                }

            }

            // I start watching the event event-type on the document.
            function startWatchingEvent(eventType) {

                $document.on(eventType, handleEvent);

            }

            // I stop watching the event event-type on the document.
            function stopWatchingEvent(eventType) {

                $document.off(eventType, handleEvent);
            }
        }
    }

})();
