(function () {
    'use strict';

    angular
        .module('app.core')
        .factory('Videoplayer', VideoplayerService);

    /** @ngInject */
    function VideoplayerService(STA) {
        function Videoplayer(options) {
            this.state = this.STATE_NOT_INIT;
            this.options = _.merge({
                bufferSize: (400 * 1024),
                drm: {
                    heartbeatPeriod: 30
                }
            }, options);
            this._playbackRate = 1;
        }

        Videoplayer.prototype.STATE_ERROR = -1;
        Videoplayer.prototype.STATE_NOT_INIT = 0;
        Videoplayer.prototype.STATE_INIT = 1;
        Videoplayer.prototype.STATE_STOP = 2;
        Videoplayer.prototype.STATE_PLAY = 3;
        Videoplayer.prototype.STATE_PAUSE = 4;

        Videoplayer.prototype.state = Videoplayer.prototype.STATE_NOT_INIT;
        Videoplayer.prototype.states = {};
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_ERROR] = 'error';
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_NOT_INIT] = 'not_init';
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_INIT] = 'init';
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_STOP] = 'stop';
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_PLAY] = 'play';
        Videoplayer.prototype.states[Videoplayer.prototype.STATE_PAUSE] = 'pause';

        Videoplayer.prototype.getPlugin = function () {
            if (!this.plugin) {
                this.plugin = this.getContainer();
                this.plugin.setAttribute('playCount', '1');
                this.plugin.setAttribute('preBufferingTime', '5');
            }

            if (!this.plugin) {
                throw new Error('player not found');
            }

            return this.plugin;
        };

        Videoplayer.prototype.init = function () {
            this.getContainer();
            var plugin = this.getPlugin();
            var self = this;

            var ended = function () {
                self.emit('ended');
                self.stop();
            };

            var pause = function () {
                self.emit('pause');
                self.pause();
            };

            var waiting = function () {
                self.emit('waiting');
            };

            var playing = function () {
                self.emit('playing');
            };

            var error = function () {
                self.emit('error');
            };

            var timeupdate = function () {
                self.emit('timeupdate', self.getCurrentTime());
            };

            var durationchange = function () {
                self.emit('durationchange', self.getDuration());
            };

            var play = function () {
                self.emit('play');
            };

            var loadedmetadata = function () {
                self.emit('loadmetadata');
                self.emit('info', self.getInfo());
            };

            var loadeddata = function () {
                self.emit('info', self.getInfo());
            };

            var lastTime = 0;
            var bufferingStart = false;
            var progress = function () {
                var seconds = self.getCurrentTime();
                var isWait = lastTime === seconds;
                lastTime = seconds;

                if (isWait) {
                    if (!bufferingStart) {
                        bufferingStart = true;
                        self.emit('bufferingstart');
                    }

                    var l = self.getPlugin().buffered.length;
                    var loadStartPercentage, loadEndPercentage;

                    while (l--) {
                        loadStartPercentage = self.getPlugin().buffered.start(l) / self.getPlugin().duration;
                        loadEndPercentage = self.getPlugin().buffered.end(l) / self.getPlugin().duration;
                        self.emit('buffering', loadEndPercentage - loadStartPercentage);
                    }

                    return;
                }

                if (bufferingStart) {
                    bufferingStart = false;
                    self.emit('bufferingend');
                }
            };

            plugin.addEventListener('ended', ended);
            plugin.addEventListener('pause', pause);
            plugin.addEventListener('waiting', waiting);
            plugin.addEventListener('playing', playing);
            plugin.addEventListener('error', error);
            plugin.addEventListener('timeupdate', timeupdate);
            plugin.addEventListener('durationchange', durationchange);
            plugin.addEventListener('play', play);
            plugin.addEventListener('loadedmetadata', loadedmetadata);
            plugin.addEventListener('loadeddata', loadeddata);
            plugin.addEventListener('progress', progress);

            self.exit = function () {
                plugin.removeEventListener('ended', ended);
                plugin.removeEventListener('pause', pause);
                plugin.removeEventListener('waiting', waiting);
                plugin.removeEventListener('playing', playing);
                plugin.removeEventListener('error', error);
                plugin.removeEventListener('timeupdate', timeupdate);
                plugin.removeEventListener('durationchange', durationchange);
                plugin.removeEventListener('play', play);
                plugin.removeEventListener('loadedmetadata', loadedmetadata);
                plugin.removeEventListener('loadeddata', loadeddata);
                plugin.removeEventListener('progress', progress);
                plugin.stop();
                plugin.setAudioTrack();
                plugin.setTextTrack();
                plugin._setSource('exitErrorUrl');
                plugin.deinit();
            };

            this._setSource(this.options.url);
            this._setAutoplay(this.options.autoplay);
            this._setState(this.STATE_INIT);
        };

        Videoplayer.prototype.deinit = function () {
            var container = this.getContainer();
            container.removeChild(this.getPlugin());
            this.plugin = null;
            this._setState(this.STATE_NOT_INIT);
            return true;
        };

        Videoplayer.prototype.getContainer = function () {
            if (!this.options.containerId) {
                throw new Error('containerId undefined');
            }

            if (!this.container) {
                this.container = document.getElementById(this.getOption('containerId'));
            }

            if (!this.container) {
                throw new Error('element by containerId (' + this.getOption('containerId') + ') not found');
            }

            return this.container;
        };

        Videoplayer.prototype.setVideo = function (url, options) {
            this.setOption('url', url);
            this.setOptions(options);
            this.deinit();
            this.init();
        };

        Videoplayer.prototype.setOptions = function (options) {
            this.options = _.merge(this.options, options);
        };

        Videoplayer.prototype.setOption = function (name, value) {
            this.options[name] = value;
            return this;
        };

        Videoplayer.prototype.getOption = function (name) {
            return this.options[name];
        };

        Videoplayer.prototype.getState = function () {
            return this.state;
        };

        Videoplayer.prototype._setState = function (stateCode) {
            if (stateCode === this.state) {
                return this;
            }

            this.state = stateCode;

            var stateName = this.getStateName(stateCode);
            this.emit('statechange', stateName);
            return this;
        };

        Videoplayer.prototype.getStateName = function (stateCode) {
            if (this.states[stateCode]) {
                return this.states[stateCode];
            }

            throw new Error('undeclared state code: ' + stateCode);
        };

        Videoplayer.prototype.getCurrentTime = function () {
            return this.timeToMseconds(this.getPlugin().currentTime);
        };

        Videoplayer.prototype.getDuration = function () {
            return Math.round((this.getPlugin().duration * 1000));
        };

        Videoplayer.prototype.getAudioTracks = function () {
            return this.getPlugin().audioTracks || [];
        };

        Videoplayer.prototype.setAudioTrack = function (track) {
            var tracks = this.getAudioTracks();
            if (tracks.length && typeof track === 'number') {
                for (var i = 0; i < tracks.length; i++) {
                    this.getPlugin().audioTracks[i].enabled = false;
                }

                if (typeof track === 'number') {
                    this.getPlugin().audioTracks[track].enabled = true;
                }
            }

            return this;
        };

        Videoplayer.prototype.getTextTracks = function () {
            return this.getPlugin().textTracks || [];
        };

        Videoplayer.prototype.setTextTrack = function (track) {
            var tracks = this.getTextTracks();
            if (tracks.length) {
                for (var i = 0; i < tracks.length; i++) {
                    this.getPlugin().textTracks[i].mode = 'hidden';
                }

                if (typeof track === 'number') {
                    this.getPlugin().textTracks[track].mode = 'showing';
                }
            }

            return this;
        };

        Videoplayer.prototype.pause = function () {
            this.getPlugin().pause();
            this._setState(this.STATE_PAUSE);
            return true;
        };

        Videoplayer.prototype.stop = function () {
            var plugin = this.getPlugin();
            plugin.pause();
            plugin.currentTime = 0;
            this._setState(this.STATE_STOP);
            return true;
        };

        Videoplayer.prototype.resume = function () {
            this.getPlugin().play();
            this._setState(this.STATE_PLAY);
            return true;
        };

        Videoplayer.prototype.stepForward = function (ms) {
            this.setCurrentTime(this.getCurrentTime() + ms);
        };

        Videoplayer.prototype.stepBackward = function (ms) {
            this.setCurrentTime(this.getCurrentTime() - ms);
        };

        Videoplayer.prototype.setCurrentTime = function (msec) {
            this.getPlugin().currentTime = this.msecondsToPlugin(msec);
            return true;
        };

        Videoplayer.prototype.play = function () {
            this.getPlugin().play();
            this._setState(this.STATE_PLAY);
            return true;
        };

        Videoplayer.prototype.startPlayback = function () {
            var plugin = this.getPlugin();
            plugin.load();
            plugin.play();
            this._setState(this.STATE_PLAY);
            return true;
        };

        Videoplayer.prototype.setPlaybackSpeed = function (speed) {
            var rate = parseFloat(speed).toFixed(1);

            this.getPlugin().playbackRate = rate;
        };

        Videoplayer.prototype.setPlaybackRate = function (speed) {
            this._playBackRate = speed;
            this.emit('ratechange', this._playBackRate);
            var rate = parseFloat(this._playBackRate).toFixed(1);
            this.getPlugin().playbackRate = rate;
        };

        Videoplayer.prototype.getPlaybackRate = function () {
            return this._playbackRate;
        };

        Videoplayer.prototype.setScreenSize = function () {
            throw new Error('setScreenSize not implemented');
        };

        Videoplayer.prototype.requestFullscreen = function () {
            var windowSize = this.getWindowSize();
            var plugin = this.getPlugin();

            plugin.width = windowSize[0];
            plugin.height = windowSize[1];

            plugin.style.position = 'fixed';
            plugin.style.top = '0px';
            plugin.style.left = '0px';
            plugin.style.width = windowSize[0] + 'px';
            plugin.style.height = windowSize[1] + 'px';
            return true;
        };

        Videoplayer.prototype.exitFullscreen = function () {
            var plugin = this.getPlugin();

            plugin.width = this.getOption('width');
            plugin.height = this.getOption('height');

            plugin.style.position = 'relative';
            plugin.style.width = plugin.width + 'px';
            plugin.style.height = plugin.height + 'px';
            return true;
        };

        Videoplayer.prototype._setSource = function (sourceUrl) {
            var source = document.createElement('source');
            var mimeType = STA.mime(sourceUrl);

            source.setAttribute('src', sourceUrl);
            source.setAttribute('type', mimeType);
            this.getPlugin().appendChild(source);
        };

        Videoplayer.prototype._setAutoplay = function (autoplay) {
            this.getPlugin().setAttribute('autoplay', autoplay);
        };

        Videoplayer.prototype.getWindowSize = function () {
            var w = window,
                d = document,
                e = d.documentElement,
                g = d.getElementsByTagName('body')[0],
                width = w.innerWidth || e.clientWidth || g.clientWidth,
                height = w.innerHeight || e.clientHeight || g.clientHeight;

            return [width, height];
        };

        Videoplayer.prototype.getDeviceScreenSize = function () {
            var width = window.screen.availWidth,
                height = window.screen.availHeight;

            return [width, height];
        };

        Videoplayer.prototype.getInfo = function () {
            var plugin = this.getPlugin();
            return {
                duration: this.getDuration(),
                currentTime: this.getCurrentTime(),
                bufferedLenght: plugin.buffered.length,
                bufferedStart: plugin.buffered.start,
                bufferedEnd: plugin.buffered.end
            };
        };

        Videoplayer.prototype.msecondsToPlugin = function (mseconds) {
            return mseconds / 1000;
        };

        Videoplayer.prototype.timeToMseconds = function (time) {
            return Math.round(time * 1000);
        };

        Videoplayer.prototype.on = function (event, callback) {
            if (!this.listeners) {
                this.listeners = {};
            }

            if (typeof this.listeners[event] === 'undefined') {
                this.listeners[event] = [];
            }

            if (typeof callback !== "function") {
                callback = function () {
                }
            }

            this.listeners[event].push(callback);
        };

        Videoplayer.prototype.off = function (event, callback) {
            if (!this.listeners) {
                this.listeners = {};
            }

            var index = this.listeners[event].indexOf(callback);
            if (index !== 0) {
                this.listeners[event].splice(index, 1);
            }
        };

        Videoplayer.prototype.emit = function (event) {
            if (!this.listeners) {
                return;
            }
            var listeners = this.listeners[event];

            if (typeof listeners === 'undefined' || listeners.length == 0) {
                return;
            }

            var args = Array.prototype.slice.call(arguments, 1);

            for (var i = 0; i < listeners.length; i++) {
                listeners[i].apply(listeners[i], args);
            }
        };

        return Videoplayer;
    }
}());
