(function () {
    'use strict';

    angular
        .module('app.core')
        .factory('STA', staHelperService);

    /** @ngInject */
    function staHelperService(uuid, localStorageService, $log, $window) {
        // Private variables

        var service = {
            init: function () {
                this.keys = this.getKeys();
                this.capabilities = this.getCapabilities();
                this.platform = this.getPlatform();
                this.settings = this.getSettings();
                this.compatible = this.isCompatible();
                this.mime = this.getMimeType;

                return this;
            },
            hasSTAApi: function () {
                return typeof SmartTvA_API !== 'undefined';
            },
            isCompatible: function () {
                var platform = this.getPlatform();

                return platform.version == '5.0.0' || platform.version == '4.0.0' || platform.version == '3.0.0';
            },
            getMimeType: function (url) {
                switch (true) {
                    case /Manifest/.test(url):
                        return 'application/vnd.ms-sstr+xml';
                        break;
                    case /.mpd/.test(url):
                        return 'application/dash+xml';
                        break;
                    default:
                        return 'video/mp4'
                        break;
                }
            },
            getKeys: function () {
                var platform = this.getPlatform();

                var keys = {};
                var keyConstant = {
                    VK_ENTER: 13,
                    VK_PAUSE: 19,
                    VK_LEFT: 37,
                    VK_UP: 38,
                    VK_RIGHT: 39,
                    VK_DOWN: 40,
                    VK_0: 48,
                    VK_1: 49,
                    VK_2: 50,
                    VK_3: 51,
                    VK_4: 52,
                    VK_5: 53,
                    VK_6: 54,
                    VK_7: 55,
                    VK_8: 56,
                    VK_9: 57,
                    VK_RED: 403,
                    VK_GREEN: 404,
                    VK_YELLOW: 405,
                    VK_BLUE: 406,
                    VK_REWIND: 412,
                    VK_STOP: 413,
                    VK_PLAY: 415,
                    VK_FAST_FWD: 417,
                    VK_BACK: 461,
                    VK_PREV: 33,
                    VK_NEXT: 34
                };

                if (platform.manufacturer = 'PHILIPS' && platform.version == "3.0.0") {
                    keyConstant.VK_BACK = 8;
                }

                for (var k in keyConstant) {
                    try {
                        keys[k] = (typeof $window[k] !== 'undefined') ? $window[k] : keyConstant[k];
                    } catch (e) {
                        $log.error(k + ' not defined.');
                    }
                }

                return keys;
            },
            getCapabilities: function () {
                if (!this.hasSTAApi()) {
                    return null;
                }

                var smarttv_capabilities = {};

                var platform = this.getPlatform();

                // test if the SmartTvA_API object exists
                if (platform.version == '5.0.0' || platform.version == '4.0.0' || platform.version == '3.0.0') {

                    // determine capabilities for version 3.0.0+ compliant device

                    // test 3d support
                    smarttv_capabilities.d3support = SmartTvA_API.hasCapability('3DSupport');

                    // test numerickeys support
                    smarttv_capabilities.numerickeys = SmartTvA_API.hasCapability('Key', 'numerickeys');

                    // test colorkeys support
                    smarttv_capabilities.colorkeys = SmartTvA_API.hasCapability('Key', 'colorkeys');

                    // test AllJoyn multiscreen supportaccording to specification 4.0.0+
                    smarttv_capabilities.alljoyn = SmartTvA_API.hasCapability('Multiscreen', 'AllJoyn');

                    // test Playready + MPEG-DASH combination DRM supportaccording to specification 4.0.0
                    smarttv_capabilities.playready_dash = SmartTvA_API.hasCapability('DRM', 'PlayReady', 'DASH');

                    // test Widevine + Adaptive Streaming combination DRM supportaccording to specification 4.0.0
                    smarttv_capabilities.widevine_adaptive = SmartTvA_API.hasCapability('DRM', 'Widevine', 'AdaptiveStreaming');

                    // test Multi track audio support
                    smarttv_capabilities.multiaudio = SmartTvA_API.hasCapability('Multiaudio');

                    // test TTML inband supportaccording to specification 4.0.0+
                    smarttv_capabilities.ttml_inband = SmartTvA_API.hasCapability('TTML', 'inband');

                    // test TTML out-of-band support according to specification 4.0.0+
                    smarttv_capabilities.ttml_outofband = SmartTvA_API.hasCapability('TTML', 'outofband');

                    // test Ultra High Definition support according to specification 4.0.0+
                    smarttv_capabilities.ultrahd = SmartTvA_API.hasCapability('UHD');

                    // for version 4.0+ specific devicesonly
                    smarttv_capabilities.ultrahd_maintier_51 = SmartTvA_API.hasCapability('UHD', 'MainTier', '5.1');

                    smarttv_capabilities.ultrahd_hls = SmartTvA_API.hasCapability('UHD', 'HLS');
                    smarttv_capabilities.nsd = SmartTvA_API.hasCapability('NSD');
                    smarttv_capabilities.nsd_zeroconf = SmartTvA_API.hasCapability('NSD', 'Zeroconf');
                    smarttv_capabilities.nsd_dial = SmartTvA_API.hasCapability('NSD', 'DIAL');
                    smarttv_capabilities.ememse = SmartTvA_API.hasCapability('EMEMSE');
                    smarttv_capabilities.ememse_widevine = SmartTvA_API.hasCapability('EMEMSE', 'Widevine');
                }

                return smarttv_capabilities;
            },
            getSettings: function () {
                var smarttv_settings = {};

                // Get a uuid
                smarttv_settings.uuid = uuid.v4();

                // Check if localstorage and cookie is supported
                smarttv_settings.localStorage = localStorageService.isSupported;
                smarttv_settings.cookie = localStorageService.cookie.isSupported;

                return smarttv_settings;
            },
            getPlatform: function () {
                var userAgent = navigator.userAgent;
                var smarttv_platform = {};

                smarttv_platform.ua = userAgent;
                smarttv_platform.manufacturer = 'unknown';
                smarttv_platform.type = 'unknown';
                smarttv_platform.version = 'unknown';

                var substr_pos;

                // check if this is a Smart TV Alliance 2.5+ compliant platform
                substr_pos = userAgent.search(/SmartTvA/i);
                if (substr_pos > -1) {
                    // this is a Smart TV Alliance 2.5+ compliant platform with a new UA identification
                    smarttv_platform.type = userAgent;
                    smarttv_platform.manufacturer = 'Smart TV Alliance';
                    smarttv_platform.version = userAgent.substr(substr_pos + 9, 5);
                    return smarttv_platform;
                }

                // for STA Specification 2.0 (or older) devices, the older identification method is used as shown below (this example is not exhaustive for all STA manufacturers):

                // check if this is the Smart TV Alliance SDK
                substr_pos = userAgent.search(/Chromium\/18/i);
                if (substr_pos > -1) {
                    substr_pos = userAgent.search(/Linux i686/i);
                    if (substr_pos > -1) {
                        smarttv_platform.type = userAgent;
                        smarttv_platform.manufacturer = 'Smart TV Alliance SDK';
                        smarttv_platform.version = '2.0';
                    }
                }

                // check for specific manufacturers (overriding STA SDK option)
                substr_pos = userAgent.search(/NetCast.TV-2013/i);
                if (substr_pos > -1) {
                    smarttv_platform.type = userAgent.substr(substr_pos, 15);
                    smarttv_platform.manufacturer = 'LG';
                    smarttv_platform.version = '2.0';
                }
                substr_pos = userAgent.search(/NetCast.TV-2012/i);
                if (substr_pos > -1) {
                    smarttv_platform.type = userAgent.substr(substr_pos, 15);
                    smarttv_platform.manufacturer = 'LG';
                    smarttv_platform.version = '1.0';
                }
                else {
                    substr_pos = userAgent.search(/NetCast.Media/i);
                    if (substr_pos > -1) {
                        smarttv_platform.type = userAgent.substr(substr_pos, 18);
                        smarttv_platform.manufacturer = 'LG';
                    }
                    else {
                        substr_pos = userAgent.search(/NetCast/i);
                        if (substr_pos > -1) {
                            smarttv_platform.type = userAgent.substr(substr_pos, 12);
                            smarttv_platform.manufacturer = 'LG';
                        }
                    }
                }
                substr_pos = userAgent.search(/NETTV\/4/i);
                if (substr_pos > -1) {
                    smarttv_platform.type = userAgent.substr(substr_pos, 11);
                    smarttv_platform.manufacturer = 'PHILIPS';
                    smarttv_platform.version = '1.0';
                }
                substr_pos = userAgent.search(/NETTV\/4.2/i);
                if (substr_pos > -1) {
                    smarttv_platform.type = userAgent.substr(substr_pos, 11);
                    smarttv_platform.manufacturer = 'PHILIPS';
                    smarttv_platform.version = '2.0';
                }
                else {
                    substr_pos = userAgent.search(/NETTV/i);
                    if (substr_pos > -1) {
                        smarttv_platform.type = userAgent.substr(substr_pos, 11);
                        smarttv_platform.manufacturer = 'PHILIPS';
                    }
                }
                substr_pos = userAgent.search(/TOSHIBA-DTV/i);
                if (substr_pos > -1) {
                    smarttv_platform.type = userAgent.substr(substr_pos, 11);
                    smarttv_platform.manufacturer = 'TOSHIBA';
                    smarttv_platform.version = '2.0';
                }
                return smarttv_platform;
            },
            close: function () {
                // test if the SmartTvA_API object exists
                if (!this.hasSTAApi()) {
                    window.close()
                    return false;
                }

                var platform = this.getPlatform();
                if (platform.version == '5.0.0' || platform.version == '4.0.0' || platform.version == '3.0.0') {
                    SmartTvA_API.exit(); // control is handed back to the devicefrom this point
                    return true;
                }

                return false;
            }
        };

        return service.init();
    }
}());
